@@ -1,10 +1,11 @@
 /****************************************************************************
- *
- * Copyright (c) 1996-2021 Vladi Belperchinov-Shabanski "Cade" 
- * http://cade.noxrun.com/  <cade@noxrun.com> <cade@bis.bg>
- *
- * SEE `README',`LICENSE' OR `COPYING' FILE FOR LICENSE AND OTHER DETAILS!
- *
+ #
+ # Copyright (c) 1996-2023 Vladi Belperchinov-Shabanski "Cade" 
+ # https://cade.noxrun.com/  <cade@noxrun.com> <cade@bis.bg>
+ # https://cade.noxrun.com/projects/vfu     https://github.com/cade-vs/vfu
+ #
+ # SEE `README',`LICENSE' OR `COPYING' FILE FOR LICENSE AND OTHER DETAILS!
+ #
  ****************************************************************************/
 
 #include "vfudir.h"
@@ -16,6 +17,8 @@
 #include "vfumenu.h"
 
 VArray size_cache;
+int    size_cache_count;
+int    size_cache_sorted_by_name;
 
 /*###########################################################################*/
 
@@ -65,7 +68,7 @@ VArray size_cache;
       }
   }
 
-int vfu_get_dir_name( const char *prompt, VString &target, int should_exist, int type )
+int vfu_get_dir_name( const char *prompt, VString &target_in, int should_exist, int type )
 {
   int res = -1;
   /*
@@ -75,26 +78,26 @@ int vfu_get_dir_name( const char *prompt, VString &target, int should_exist, int
   */
   VArray dir_list;
 
-  say1(prompt);
-  say2("");
+  say1( prompt );
+  say2( ""     );
 
-  int pos = 0;
+  int pos  = 0;
   int page = 0;
-  int ch = 0;
+  int wch  = 0;
 
   int insert = 1;
   int firsthit = 1;
 
+  WString target = target_in;
   pos = str_len( target );
 
   //------------------------------------------------------------------
-
   con_cshow();
-  say2( target, firsthit ? cINPUT2 : cINPUT );
+  // say2( target, firsthit ? cINPUT2 : cINPUT );
   while(1)
     {
     int mx = con_max_x() - 1;
-    VString target_out = target;
+    WString target_out = target;
     if ( (pos < page) || (pos+1 > page + mx) || (page > 0 && pos == page) )
       page = pos - mx / 2;
     if ( page < 0 ) page = 0;
@@ -103,49 +106,47 @@ int vfu_get_dir_name( const char *prompt, VString &target, int should_exist, int
     str_sleft( target_out, mx );
     str_pad( target_out, -mx );
     if ( page > 0 )
-      str_set_ch( target_out, 0, '<' );
+      str_set_ch( target_out, 0, L'<' );
     if ( str_len( target ) - page > mx )
-      str_set_ch( target_out, mx-1, '>' );
+      str_set_ch( target_out, mx-1, L'>' );
 
-    say2( target_out, firsthit ? cINPUT2 : cINPUT );
+    say2( VString( target_out ).data(), firsthit ? cINPUT2 : cINPUT );
     con_xy( pos-page+1, con_max_y() );
-
-
-    if (ch == 0) ch = con_getch();
-    if (ch == '\\') ch = '/'; /* dos hack :)) */
-    if ( ch == '/' && str_find( target, '/' ) == -1 && target[0] == '~' )
+    if ( wch == 0 ) wch = con_getwch();
+    if ( wch == L'\\') wch = L'/'; /* dos hack :)) */
+    if ( wch == L'/' && str_find( target, L'/' ) == -1 && target[0] == L'~' )
       {
-      target = tilde_expand( target );
+      target.set( tilde_expand( VString( target ) ) );
       str_fix_path( target );
       pos = str_len( target );
-      ch = 0;
+      wch = 0;
       }
 
-    if ( ( ch == 8 || ch == KEY_BACKSPACE ) && pos > 0 )
+    if ( ( wch == 8 || wch == UKEY_BACKSPACE ) && pos > 0 )
       {
       pos--;
       str_del( target, pos, 1 );
       }
     else
-    if (ch == KEY_CTRL_A && str_len( target ) > 0)
+    if ( wch == UKEY_CTRL_A && str_len( target ) > 0)
       {
       int z = str_len( target )-1;
-      if ( str_get_ch(target, z) == '/' ) z--;
-      while ( z > 0 && str_get_ch(target,z) != '/' ) z--;
+      if ( str_get_ch(target, z) == L'/' ) z--;
+      while ( z > 0 && str_get_ch(target,z) != L'/' ) z--;
       z++;
       str_sleft(target,z);
       pos = z;
       }
     else
-    if ( ch == 9 && str_len( target ) > 0 )
+    if ( wch == 9 && str_len( target ) > 0 )
       {
-      int z;
+      int z = -1;
       dir_list.undef();
       VString dmain; /* main/base path */
       VString dtail; /* item that should be expanded/glob */
 
-      dmain = str_file_path( target );
-      dtail = str_file_name_ext( target );
+      dmain.set( str_file_path( target ) );
+      dtail.set( str_file_name_ext( target ) );
 
       /*
       int lastslash = str_rfind(target, '/');
@@ -163,29 +164,30 @@ int vfu_get_dir_name( const char *prompt, VString &target, int should_exist, int
         }
       */
 
+      int dtlen = str_len( dtail );
+      
       __glob_gdn( dmain, dtail, dir_list, type );
 
-      z = dir_list.count()-1;
       if (dir_list.count())
         {
         if ( dir_list.count() > 1)
           {
-          int li = 0; /* counter */
-          int ll = 0; /* longest directory entry */
-          int xm = 0; /* exact match entry  */
+          int li = 0; // counter 
+          int ll = 0; // longest directory entry 
+          int xm = 0; // exact match entry  
           for ( li = 0; li < dir_list.count(); li++ )
             {
             int len = strlen( dir_list[li] );
             if( len > ll )
               ll = len;
-            VString tmp1;
-            if( dtail != str_copy( tmp1, dir_list[li], 0, str_len( dtail ) ) )
+            VString current_dtail;
+            if( dtail != str_copy( current_dtail, dir_list[li], 0, dtlen ) )
               continue;
             xm = li;
             break;
             }
-          int mc = 0; /* match count        */
-          int mi = 0; /* match letter index */
+          int mc = 0; // match count        
+          int mi = dtlen; // match letter index 
           while(4)
             {
             mc = 0;
@@ -204,36 +206,56 @@ int vfu_get_dir_name( const char *prompt, VString &target, int should_exist, int
             if ( mc != dir_list.count() )
               break;
             mi++;
+//            if( dir_list[xm][mi] == 0 || dir_list[li][mi] == 0 )
+//              break;
             if( mi >= ll )
               break;
             }
-          target.setn( dmain + dir_list[xm], str_len( dmain ) + mi );
+          
+          VString exact_dtail_max;
+          str_copy( exact_dtail_max, dir_list[xm], 0, mi );
+          
+          target = dmain + exact_dtail_max;
           pos = str_len( target );
-          say2( target, cINPUT );
+          say2( VString( target ), cINPUT );
           con_xy( pos+1, con_max_y() );
 
           vfu_beep();
-          ch = con_getch();
-          if ( ch != 9 ) { dir_list.undef(); continue; }
+          wch = con_getwch();
+          if ( wch != 9 ) { dir_list.undef(); continue; }
           dir_list.sort();
+          WArray w_dir_list;
+          int di;
+          for( di = 0; di < dir_list.count(); di++ ) 
+            {
+            WString ws;
+            ws = dir_list[di];
+            w_dir_list.push( ws );
+            }
           con_chide();
-          z = vfu_menu_box( 10, 5, "Complete...", &dir_list );
+          z = vfu_menu_box( 10, 5, L"Complete...", &w_dir_list );
           con_cshow();
-          ch = 0;
+          wch = 0;
           }
         else
-          ch = 0;
-        if ( z != -1 )
           {
-          while( str_len( target ) > 0 && target[-1] != '/' )
-            str_chop( target );
+          z = 0;
+          wch = 0;
+          }
+        if ( z >= 0 )
+          {
+          int sp = str_rfind( target, L'/' );
+          if( sp < 0 ) 
+            target.undef();
+          else
+            str_copy( target, target, 0, sp + 1 );  
           target += dir_list[z];
           }
 
         pos = str_len( target );
 
         dir_list.undef();
-        if (ch != 0) continue;
+        if ( wch != 0 ) continue;
         }
       else
         { /* no match found -- cannot complete */
@@ -241,87 +263,115 @@ int vfu_get_dir_name( const char *prompt, VString &target, int should_exist, int
         }
       }
     else
-    if (ch == 13)
+    if ( wch == 13 )
       {
       res = 1;
       break;
       }
     else
-    if (ch == 27)
+    if ( wch == 27 )
       {
-      target = "";
+      target.undef();
       res = 0;
       break;
       }
-    if (ch == KEY_CTRL_U)
+    if ( wch == UKEY_CTRL_U )
       {
-      target = "";
+      target.undef();
       pos = 0;
       }
     else
-    if (ch == KEY_CTRL_X)
+    if ( wch == UKEY_CTRL_X )
       {
-        fname_t t;
-        if ( target[0] == '~' )
-          target = tilde_expand( target );
-        expand_path( target, t );
-        str_fix_path( t );
-        target = t;
+        if ( target[0] == L'~' )
+          target.set( tilde_expand( VString( target ) ) );
+
+        target.set( expand_path( VString( target ) ) );
+        str_fix_path( target );
         pos = str_len( target );
       }
     else
-    if ( ch >= 32 && ch <= 255 && ch != KEY_BACKSPACE ) // && pos < 70)
+    if ( wch >= 32 && wch != 8 && wch != UKEY_BACKSPACE && ( ! UKEY_IS_WIDE_CTRL( wch ) ) )
       {
-      if (firsthit)
+      if( firsthit )
         {
-        target = "";
+        target.undef();
         pos = 0;
         }
-      if (!insert) str_del( target, pos, 1 );
-      str_ins_ch( target, pos, ch );
+      if ( ! insert ) str_del( target, pos, 1 );
+      str_ins_ch( target, pos, wch );
       pos++;
       } else
-    if( ch == KEY_LEFT  )
+    if( wch == UKEY_LEFT  )
       {
       if (pos > 0)
         pos--;
       } else
-    if( ch == KEY_RIGHT )
+    if( wch == UKEY_RIGHT )
       {
       if (pos < str_len( target ))
         pos++;
       } else
-    if ( ch == KEY_IC   ) insert = !insert; else
-    if ( ch == KEY_HOME ) pos = 0; else
-    if ( ch == KEY_END  ) pos = str_len(target); else
-    if ( ch == KEY_DC  && pos < str_len(target) )
-       str_del( target, pos, 1 ); else
-    if ( ch == KEY_NPAGE || ch == KEY_PPAGE )
+    if ( wch == UKEY_INS  ) insert = !insert; else
+    if ( wch == UKEY_HOME ) pos = 0; else
+    if ( wch == UKEY_END  ) pos = str_len(target); else
+    if ( wch == UKEY_DEL  && pos < str_len(target) ) str_del( target, pos, 1 ); else
+    if ( wch == UKEY_PGUP || wch == UKEY_PGDN )
       {
       con_chide();
-      int zz = vfu_hist_menu( 5, 5, ( ch == KEY_PPAGE ) ? "Dir Entry History" : "ChDir History",
-                                    ( ch == KEY_PPAGE ) ? HID_GETDIR : HID_CHDIR );
+      int zz = vfu_hist_menu( 5, 5, ( wch == UKEY_PGUP ) ? L"Dir Entry History" : L"ChDir History",
+                                    ( wch == UKEY_PGUP ) ? HID_GETDIR : HID_CHDIR );
       con_cshow();
       if (zz != -1)
         {
-        const char* pc = vfu_hist_get( ( ch == KEY_PPAGE ) ? HID_GETDIR : HID_CHDIR, zz );
+        const char* pc = vfu_hist_get( ( wch == UKEY_PGUP ) ? HID_GETDIR : HID_CHDIR, zz );
         if ( pc )
           {
           target = pc;
           pos = str_len( target );
           }
         }
+      } else
+    if ( wch == UKEY_CTRL_O  )
+      {
+      mb.undef();
+      mb.push( L"A Add current DATE+TIME" );
+      mb.push( L"D Add current DATE only" );
+      mb.push( L"N Add current directory name" );
+      vfu_menu_box( 20, -10, L"  Directory helper" );
+      int ec = menu_box_info.ec;
+      if( ec == 'A' or ec == 'D' )
+        {
+        char   time_str[32];
+        time_t timenow = time( NULL );
+        tm     tmnow;
+        localtime_r( &timenow, &tmnow );
+        if( strftime( time_str, sizeof(time_str) - 1, ec == L'A' ? "%Y%m%d_%H%M%S" : "%Y%m%d", &tmnow ) )
+          {
+          if( target[-1] != '/' ) target += "_";
+          target += time_str;
+          }
+        } else
+      if( ec == 'N' )
+        {
+        VString str = work_path;
+        str_cut_right( str, "/" );
+        str = str_file_name_ext( str );
+        str_cut_right( target, L"/" );
+        target += "/" + WString( str );
+        }
+      pos = str_len( target );
       }
-    ch = 0;
+    wch = 0;
     firsthit = 0;
     }
   con_chide();
 
   //------------------------------------------------------------------
   str_cut_spc( target );
-  if ( res == 1 && target[0] == '~' )
+  if ( res == 1 && target[0] == L'~' )
     {
-    target = tilde_expand( target );
+    target.set( tilde_expand( VString( target ) ) );
     str_fix_path( target );
     }
   /*
@@ -329,42 +379,42 @@ int vfu_get_dir_name( const char *prompt, VString &target, int should_exist, int
   leaveok(stdscr, TRUE);
   #endif
   */
-  if ( res == 1 && str_len( target ) > 0 && should_exist && type == 'D' && !dir_exist( target ))
+  if ( res == 1 && str_len( target ) > 0 && should_exist && type == 'D' && !dir_exist( VString( target ) ))
     {
     vfu_beep();
-    int ch = tolower( vfu_ask( "Directory does not exist! Create? "
-                               "( ENTER=Yes, ESC=cancel )",
-                               "\033\rcC" ));
-    if ( ch == 27 )
+    wchar_t wch = towlower( vfu_ask( L"Directory does not exist! Create? "
+                                     L"( ENTER=Yes, ESC=cancel )",
+                                     L"\r" ));
+    if ( wch == 27 )
       {
       res = 0;
       target = "";
       }
-    else
-    if ( ch == 13 )
-       if (make_path( target ))
+    else if ( wch == 13 )
+       if ( make_path( VString( target ) ))
          {
-         if(tolower(
-            vfu_ask( "Cannot create path! ( ESC=cancel, C=continue-anyway )",
-            "\033Cc" )) == 27)
+         if(vfu_ask( L"Cannot create path! ( ESC=cancel, C=continue-anyway )", L"Cc" ) == 27 )
             {
             res = 0;
-            target = "";
+            target.undef();
             }
          }
     }
 
   say1(" ");
   say2(" ");
-  if ( str_len( target ) > 0)
-    {
-    if( file_is_dir( target ) )
-      str_fix_path( target );
-    vfu_hist_add( HID_GETDIR, target );
-    }
 
   str_cut_spc( target );
 
+  if ( str_len( target ) > 0 )
+    {
+    if( file_is_dir( VString( target ) ) )
+      str_fix_path( target );
+    vfu_hist_add( HID_GETDIR, VString( target ) );
+    }
+
+  target_in = target;
+
   ASSERT( res == 0 || res == 1 );
   return res;
 }
@@ -408,7 +458,7 @@ void vfu_chdir( const char *a_new_dir )
       z = tree_find( target, &mb );
       if (z > 1)
         {
-        z = vfu_menu_box( 10, 5, "Change dir to..." );
+        z = vfu_menu_box( 10, 5, L"Change dir to..." );
         if (z > -1)
           target = mb.get(z);
         else
@@ -420,21 +470,7 @@ void vfu_chdir( const char *a_new_dir )
       }
   VString str = target;
   str_cut_spc( str );
-  #ifdef _TARGET_GO32_
-    if ( str[0] == '/' )
-      {
-      str_ins_ch( str, 0, ':' );
-      str_ins_ch( str, 0, work_path[0] );
-      } else
-    if ( str[1] == ':' && str[2] == 0 ) /* c: d: e: */
-      {
-      expand_path( str, t );
-      str = t;
-      }
-    if ( str[1] == ':' && str[2] == '/' )
-  #else /* _TARGET_GO32_ -> i.e. _TARGET_UNIX_ here*/
-    if (str[0] == '/')
-  #endif /* _TARGET_GO32_ */
+  if (str[0] == '/')
     { /* root directory */
     target = str;
     }
@@ -465,7 +501,7 @@ void vfu_chdir( const char *a_new_dir )
 
 void vfu_chdir_history()
 {
-  int z = vfu_hist_menu( 5, 5, "ChDir History", HID_CHDIR );
+  int z = vfu_hist_menu( 5, 5, L"ChDir History", HID_CHDIR );
   if (z == -1) return;
   do_draw = 1;
   //strcpy(opt.LastDir, CPath);
@@ -567,11 +603,7 @@ fsize_t __tree_rebuild_process( const char* path )
 
     if (is_link) continue;
 
-    #ifdef _TARGET_GO32_
-    dosstat(dir, &st);
-    #else
     stat(new_name, &st);
-    #endif
 
     int is_dir = S_ISDIR(st.st_mode);
 
@@ -607,7 +639,7 @@ fsize_t __tree_rebuild_process( const char* path )
       }
     else
       { /* file */
-      size += file_st_size( &st );
+      size += st.st_size;
       }
 
     }
@@ -620,13 +652,6 @@ fsize_t __tree_rebuild_process( const char* path )
 
 void tree_rebuild()
 {
-#ifdef _TARGET_GO32_
-// we do need only files sizes -- so the other stuff under dos is unneeded :)
-_djstat_flags = _STAT_INODE | _STAT_EXEC_EXT | _STAT_EXEC_MAGIC |
-                _STAT_EXEC_MAGIC | _STAT_DIRSIZE | _STAT_ROOT_TIME |
-                _STAT_WRITEBIT;
-// _djstat_flags = 0;
-#endif
   dir_tree.undef();
   size_cache.undef();
   say1( "Rebuilding tree..." );
@@ -634,9 +659,6 @@ _djstat_flags = _STAT_INODE | _STAT_EXEC_EXT | _STAT_EXEC_MAGIC |
   __tree_rebuild_process( "/" );
 
   tree_fix();
-#ifdef _TARGET_GO32_
- _djstat_flags = 0;
-#endif
 
   dir_tree_changed = 1;
   tree_save();
@@ -725,7 +747,7 @@ void tree_draw_page( ScrollPos &scroll )
   str_mul( str, con_max_x() );
   str = "          SiZE DiRECTORY" + str;
   str_sleft( str, con_max_x()-16 );
-  con_out(1,3, str, cHEADER );
+  vfu_con_out(1,3, str, cHEADER );
   int z = 0;
   for(z = 0; z < scroll.pagesize(); z++)
     {
@@ -782,23 +804,23 @@ void tree_view()
   if ( new_pos == -1 )
     new_pos = 0;
 
-  BSet set; /* used for searching */
-  set.set_range1( 'a', 'z' );
-  set.set_range1( 'A', 'Z' );
-  set.set_range1( '0', '9' );
-  set.set_str1( "._-~" );
-  set.set_str1( "?*>[]" );
+  VString search_set;
+  str_add_ch_range( search_set, 'a', 'z' );
+  str_add_ch_range( search_set, 'A', 'Z' );
+  str_add_ch_range( search_set, '0', '9' );
+  search_set.cat( "._-~?*>[]" );
 
   ScrollPos scroll;
   scroll.set_min_max( 0, dir_tree.count()-1 );
-  scroll.set_pagesize( FPS + 2 );
+  scroll.set_pagesize( FLPS + 2 );
+  scroll.set_pagestep( OPT_SCROLL_PAGESTEP(opt.scroll_pagestep) );
   scroll.go( new_pos );
 
   int key = 0;
   int opos = -1;
   int opage = -1;
   while( key != 27 && key != 13 && key != '-' &&
-         toupper( key ) != 'Q' && toupper( key ) != 'X' && key != KEY_ALT_X )
+         toupper( key ) != 'Q' && toupper( key ) != 'X' && key != UKEY_ALT_X )
     {
     if ( key >= 'A' && key <= 'Z' ) key = tolower( key );
     if ( key == 's' )
@@ -806,9 +828,9 @@ void tree_view()
         str = "";
         say1( "Enter search pattern: ( use TAB to advance )" );
         key = con_getch();
-        while( set.in( key ) || key == 8 || key == KEY_BACKSPACE || key == 9 )
+        while( str_find( search_set, key ) >= 0 || key == 8 || key == UKEY_BACKSPACE || key == 9 )
           {
-          if ( key == 8 || key == KEY_BACKSPACE )
+          if ( key == 8 || key == UKEY_BACKSPACE )
             str_trim_right( str, 1 );
           else
           if ( key != 9 )
@@ -861,12 +883,12 @@ void tree_view()
     else
     switch( key )
       {
-      case KEY_UP     : scroll.up(); break;
-      case KEY_DOWN   : scroll.down(); break;
-      case KEY_PPAGE  : scroll.ppage(); break;
-      case KEY_NPAGE  : scroll.npage(); break;
-      case KEY_HOME   : scroll.home(); break;
-      case KEY_END    : scroll.end(); break;
+      case UKEY_UP    : scroll.up(); break;
+      case UKEY_DOWN  : scroll.down(); break;
+      case UKEY_PGUP  : scroll.ppage(); break;
+      case UKEY_PGDN  : scroll.npage(); break;
+      case UKEY_HOME  : scroll.home(); break;
+      case UKEY_END   : scroll.end(); break;
       case 'r'        : tree_rebuild();
                         scroll.set_min_max( 0, dir_tree.count()-1 );
                         scroll.home();
@@ -878,7 +900,7 @@ void tree_view()
                         scroll.home();
                         break;
       case 'z'        :
-      case KEY_CTRL_Z :
+      case UKEY_CTRL_Z :
                         str = dir_tree[scroll.pos()];
                         str_tr( str, "\\", "/" );
                         size_cache_set( str, vfu_dir_size( str ) );
@@ -905,18 +927,22 @@ void tree_view()
 }
 
 /*###########################################################################*/
-
-#define SIZE_CACHE_OFFSET      15
-#define SIZE_CACHE_OFFSET_STR "15"
+ 
+#define SIZE_CACHE_OFFSET       15
 #define SIZE_CACHE_OFFSET_CLEAN (SIZE_CACHE_OFFSET+8+1)
+#define SIZE_CACHE_OFFSET_STR  "15"
 
 void size_cache_load()
 {
+  size_cache_sorted_by_name = 0;
+  size_cache.set_block_size( 1024*1024 );
   size_cache.undef();
+  size_cache_count = 0;
   size_cache.fload( filename_size_cache );
   // removes old-style size cache entries
   if( size_cache.count() > 0 && ( size_cache[0][SIZE_CACHE_OFFSET] != '|' || size_cache[0][SIZE_CACHE_OFFSET_CLEAN] != '|' ) )
     size_cache.undef();
+  size_cache_count = size_cache.count();
 }
 
 void size_cache_save()
@@ -926,7 +952,27 @@ void size_cache_save()
 
 int size_cache_cmp( const char* s1, const char* s2 )
 {
-  return strcmp( s1 + SIZE_CACHE_OFFSET, s2 + SIZE_CACHE_OFFSET );
+  int off = size_cache_sorted_by_name ? SIZE_CACHE_OFFSET_CLEAN : SIZE_CACHE_OFFSET;
+/*
+  const unsigned char *ss1 = (const unsigned char *) s1 + off;
+  const unsigned char *ss2 = (const unsigned char *) s2 + off;
+  unsigned char c1, c2;
+
+  do
+    {
+    c1 = (unsigned char) *ss1++;
+    c2 = (unsigned char) *ss2++;
+    if ( c1 == '\0' )
+       return c1 - c2;
+    }
+  while (c1 == c2);
+
+  if( c1 == '/' ) return -1;
+  if( c2 == '/' ) return +1;
+
+  return c1 - c2;
+*/
+  return strcmp( s1 + off, s2 + off );
 }
 
 VString size_cache_compose_key( const char *s, fsize_t size )
@@ -934,6 +980,9 @@ VString size_cache_compose_key( const char *s, fsize_t size )
   const char *ps;
   fname_t ss;
   expand_path( s, ss );
+  str_fix_path( ss );
+  // replace / with 0x01 to properly pass strcmp (using / is wrong!)
+  str_tr( ss, "/\x0a\x0b\x0c\x0d", "\x01...." );
   ps = ss;
 
   char s_adler[16];
@@ -951,14 +1000,14 @@ VString size_cache_compose_key( const char *s, fsize_t size )
 
 int size_cache_index( const char *s )
 {
-  if ( size_cache.count() == 0 ) return -1;
+  if ( size_cache.count() == 0 || size_cache_count == 0 ) return -1;
   VString str = size_cache_compose_key( s, 0 );
   int l = 0;
-  int h = size_cache.count() - 1;
+  int h = size_cache_count - 1;
   int m = h;
   while(4)
     {
-    int r =  size_cache_cmp( size_cache[m], str );
+    int r = size_cache_cmp( size_cache[m], str );
     if ( l == m && r != 0 )
       return -1;
     if ( r > 0 )
@@ -995,39 +1044,63 @@ void size_cache_set( const char *s, fsize_t size, int sort )
     {
     size_cache.push( size_cache_compose_key( s, size ) );
     if( sort )
-      size_cache.sort( 0, size_cache_cmp );
+      size_cache_sort();
     }
 }
 
+fsize_t size_cache_get_pending( const char *s )
+{
+  VString str = size_cache_compose_key( s, 0 );
+  for( int i = size_cache_count; i < size_cache.count(); i++ )
+    {
+    if( size_cache_cmp( size_cache[i], str ) ) continue;
+    str = size_cache[i];
+    str_sleft( str, SIZE_CACHE_OFFSET );
+    return atof( str );
+    }
+  return -1;
+}
+
 // this function is used to add quickly entries to the cache
 // it REQUIRES that size_cache_sort() is called after it!
-void size_cache_append( const char *s, fsize_t size )
+void size_cache_append( const char *s, fsize_t size, int sort )
 {
   size_cache.push( size_cache_compose_key( s, size ) );
+  if( sort )
+    size_cache_sort();
 }
 
+/* deletes all *s's subdir sizes */
 void size_cache_clean( const char *s )
 {
   VString str = size_cache_compose_key( s, 0 );
-  size_t qc = str_len( str );
+  ssize_t qc = str_len( str );
   str_trim_left( str, SIZE_CACHE_OFFSET_CLEAN );
   int sl = str_len( str );
   int z = 0;
-  while( z < size_cache.count() )
+  int in = 0;
+  while( z < size_cache_count )
     {
     const char* ps = size_cache[z].data() + SIZE_CACHE_OFFSET_CLEAN;
-    if ( ( strlen( size_cache[z] ) >= qc && strncmp( ps, str, sl ) == 0 && (ps[sl] == '/' || ps[sl] == 0) )
-         ||
-         ( size_cache[z][SIZE_CACHE_OFFSET_CLEAN] != '|' ) )
+    if ( ( str_len( size_cache[z] ) >= qc && (ps[sl] == 1 || ps[sl] == 0) && strncmp( ps, str, sl ) == 0 ) ) // ps[sl] == 1 is alternative for "/" see size_cache_compose_key()
+      {
       size_cache.del( z );
+      size_cache_count--;
+      if ( size_cache_sorted_by_name ) in = 1;
+      }
     else
+      {
+      if( size_cache_sorted_by_name && in ) return;
       z++;
+      }
     }
 }
 
-void size_cache_sort()
+void size_cache_sort( int use_names )
 {
+  size_cache_sorted_by_name = !!use_names;
   size_cache.sort( 0, size_cache_cmp );
+  size_cache_count = size_cache.count();
 }
 
 /*###########################################################################*/
@@ -1043,11 +1116,7 @@ int tree_index( const char *s )
 
   if ( dir_tree.count() == 0 ) return -1;
 
-  const char *s1 = s
-  #ifdef _TARGET_GO32_
-  + 2; // to cut `d:' chars len
-  #endif
-  ;
+  const char *s1 = s;
   sl1 = strlen( s1 );
 
   z = __tree_index_last_cache + 1;
@@ -1099,7 +1168,7 @@ const char* tree_find( const char *s ) // return full path by dirname
 /*-----------------------------------------------------------------------*/
 /* return count of found dirnames and stores them to va */
 
-int tree_find( const char *s, VArray *va )
+int tree_find( const char *s, WArray *wa )
 {
   VString str;
   int z = 0;
@@ -1113,15 +1182,38 @@ int tree_find( const char *s, VArray *va )
       {
       str = dir_tree[z];
       str_tr( str, "\\", "/" );
-      va->push( str );
+      wa->push( WString( str ) );
       }
     }
-  return va->count();
+  return wa->count();
 }
 
 /*###########################################################################*/
 
-fsize_t __dir_size_process( const char* path, int mode, dev_t src_dev = 0 )
+VString DirSizeInfo::str()
+{
+  VString str;
+
+  VString ds; // dirs  count
+  VString fs; // files count
+  VString ls; // links count
+  VString ts; // files count
+
+  ds.i( dirs_count );
+  fs.i( files_count );
+  ls.i( links_count );
+  ts.fi( size );
+  vfu_str_comma( ds );
+  vfu_str_comma( fs );
+  vfu_str_comma( ls );
+  vfu_str_comma( ts );
+  
+  sprintf( str, "Dirs: %s | Files: %s | Links: %s | Size: %s ( %s )", ds.data(), fs.data(), ls.data(), fsize_fmt( size, opt.use_gib_usage ).data(), ts.data() );
+  
+  return str;
+}
+
+fsize_t __dir_size_process( const char* path, int mode, dev_t src_dev = 0, DirSizeInfo* size_info = NULL )
 {
   if ( vfu_break_op() )
     return -1;
@@ -1143,21 +1235,23 @@ fsize_t __dir_size_process( const char* path, int mode, dev_t src_dev = 0 )
     lstat(new_name, &st);
     int is_link = int(S_ISLNK(st.st_mode));
     memset( &st, 0, sizeof(st) );
-    #ifdef _TARGET_GO32_
-      dosstat(dir, &st);
-    #else
-      stat(new_name, &st);
-    #endif
+    stat(new_name, &st);
     int is_dir = int(S_ISDIR(st.st_mode));
 
-    if ( is_link && ! ( mode & DIR_SIZE_FOLLOWSYMLINKS ) ) continue; /* skip links */
+    if ( is_link )
+      {
+      if ( size_info ) size_info->links_count++;
+      if ( ! ( mode & DIR_SIZE_FOLLOWSYMLINKS ) ) continue; /* skip links */
+      } 
     if ( is_dir  )
       {
+      if ( size_info ) size_info->dirs_count++;
       if ( src_dev && src_dev != st.st_dev ) continue;
       strcat( new_name, "/" );
-      fsize_t dir_size = __dir_size_process( new_name, mode );
+      fsize_t dir_size = __dir_size_process( new_name, mode, src_dev, size_info );
       if ( dir_size == -1 )
         {
+        // break operation received, return now
         closedir(dir);
         return -1;
         }
@@ -1168,7 +1262,15 @@ fsize_t __dir_size_process( const char* path, int mode, dev_t src_dev = 0 )
         }
       }
     else
-      size += file_st_size( &st );
+      {
+      fsize_t fs = st.st_size;
+      if ( size_info ) 
+        {
+        size_info->files_count++;
+        size_info->size += fs;
+        }
+      size += fs;
+      }
 
     }
   closedir(dir);
@@ -1178,13 +1280,13 @@ fsize_t __dir_size_process( const char* path, int mode, dev_t src_dev = 0 )
   return size;
 }
 
-fsize_t vfu_dir_size( const char *s, int sort, int mode )
+fsize_t vfu_dir_size( const char *s, int sort, int mode, DirSizeInfo* size_info )
 {
-  fname_t t;
-  expand_path( s, t );
-  //size_cache_clean( t );
-  str_fix_path( t );
-  size_cache_clean( t );
+  fname_t ss;
+  expand_path( s, ss );
+  str_fix_path( ss );
+  if( ! strncmp( ss, "/proc/", 6 ) ) return 0;
+  size_cache_clean( ss );
 
   int src_dev = 0;
   if( mode & DIR_SIZE_SAMEDEVONLY )
@@ -1193,8 +1295,9 @@ fsize_t vfu_dir_size( const char *s, int sort, int mode )
     stat( s, &st);
     src_dev = st.st_dev;
     }
-  fsize_t size = __dir_size_process( t, mode, src_dev );
-  size_cache_set( t, size, sort );
+  fsize_t size = __dir_size_process( ss, mode, src_dev, size_info );
+  if( size == -1 ) return -1;
+  size_cache_append( ss, size, sort );
   return size;
 }
 
