@@ -1,10 +1,11 @@
 /****************************************************************************
- *
- * Copyright (c) 1996-2021 Vladi Belperchinov-Shabanski "Cade" 
- * http://cade.noxrun.com/  <cade@noxrun.com> <cade@bis.bg>
- *
- * SEE `README',`LICENSE' OR `COPYING' FILE FOR LICENSE AND OTHER DETAILS!
- *
+ #
+ # Copyright (c) 1996-2023 Vladi Belperchinov-Shabanski "Cade" 
+ # https://cade.noxrun.com/  <cade@noxrun.com> <cade@bis.bg>
+ # https://cade.noxrun.com/projects/vfu     https://github.com/cade-vs/vfu
+ #
+ # SEE `README',`LICENSE' OR `COPYING' FILE FOR LICENSE AND OTHER DETAILS!
+ #
  ****************************************************************************/
 
 #ifndef _VFUDIR_H_
@@ -13,6 +14,7 @@
 #include "vfu.h"
 
 extern VArray size_cache;
+extern int size_cache_count;
 
 /*###########################################################################*/
 
@@ -39,7 +41,7 @@ int  tree_index( const char *s );
 const char* tree_find( const char *s ); /* return full path by dirname */
 
 /* return count of found dirnames and stores them to sc */
-int tree_find( const char *s, VArray *va );
+int tree_find( const char *s, WArray *wa );
 
 void size_cache_load();
 void size_cache_save();
@@ -47,17 +49,36 @@ VString size_cache_compose_key( const char *s, fsize_t size );
 int size_cache_index( const char *s );
 fsize_t size_cache_get( const char *s );
 void size_cache_set( const char *s, fsize_t size, int sort = 1 );
-void size_cache_append( const char *s, fsize_t size );
+void size_cache_append( const char *s, fsize_t size, int sort = 0 );
+fsize_t size_cache_get_pending( const char *s );
 void size_cache_clean( const char *s );
-void size_cache_sort();
+void size_cache_sort( int use_names = 0 );
 
 /*###########################################################################*/
 
-#define DIR_SIZE_NORMAL         0
-#define DIR_SIZE_FOLLOWSYMLINKS 2
-#define DIR_SIZE_SAMEDEVONLY    4
+#define DIR_SIZE_NORMAL           0
+#define DIR_SIZE_FOLLOWSYMLINKS   2
+#define DIR_SIZE_SAMEDEVONLY      4
+#define DIR_SIZE_NO_CACHECLEAN  128
 
-fsize_t vfu_dir_size( const char *s, int sort = 1, int mode = DIR_SIZE_NORMAL );
+struct DirSizeInfo
+  {
+  DirSizeInfo() { reset(); };
+  void reset()  
+    {
+    dirs_count  = 0;
+    files_count = 0;
+    links_count = 0;
+    size = 0;
+    };
+  int dirs_count;
+  int files_count;
+  int links_count;
+  fsize_t size;
+  VString str();
+  };
+
+fsize_t vfu_dir_size( const char *s, int sort = 1, int mode = DIR_SIZE_NORMAL, DirSizeInfo* size_info = NULL );
 
 #endif //_VFUDRI_H_
 
