@@ -1,10 +1,11 @@
 /****************************************************************************
- *
- * Copyright (c) 1996-2021 Vladi Belperchinov-Shabanski "Cade" 
- * http://cade.noxrun.com/  <cade@noxrun.com> <cade@bis.bg>
- *
- * SEE `README',`LICENSE' OR `COPYING' FILE FOR LICENSE AND OTHER DETAILS!
- *
+ #
+ # Copyright (c) 1996-2023 Vladi Belperchinov-Shabanski "Cade" 
+ # https://cade.noxrun.com/  <cade@noxrun.com> <cade@bis.bg>
+ # https://cade.noxrun.com/projects/vfu     https://github.com/cade-vs/vfu
+ #
+ # SEE `README',`LICENSE' OR `COPYING' FILE FOR LICENSE AND OTHER DETAILS!
+ #
  ****************************************************************************/
 
 #include "vfu.h"
@@ -18,11 +19,6 @@
 
 /*---------------------------------------------------------------------------*/
 
-fsize_t file_st_size( struct stat* st )
-{
-  return st->st_size + ( st->st_size < 0 ) * ((uintmax_t)TYPE_MAXIMUM(off_t) - TYPE_MINIMUM(off_t) + 1);
-}
-
 VString vfu_readlink( const char* fname )
 {
   fname_t t;
@@ -41,72 +37,54 @@ VString out;
 VString s;
 
 int i = 0;
-int one = 0;
 int full = 0;
 
 // toggles
-#ifdef _TARGET_GO32_
-  int use_SFN = 0;
-#endif
-
 while( a_line[i] )
   {
   if ( a_line[i] == '%' )
     {
     switch( a_line[i+1] )
       {
-      #ifdef _TARGET_GO32_
-      case '_' : use_SFN = 1; break;
-      #endif
-
       case 'r' : /* rescan files after */
-      case 'R' : a_options += "r"; break; // refresh all files after (readdir)
+      case 'R' : a_options += "r"; 
+                 break; 
+
       case 'f' : /* file name */
-      case 'F' :
-      case 'g' :
-      case 'G' : one = 0;
-                 if ( a_line[i+1] == 'f' ) one = 1;
-                 if ( a_line[i+1] == 'F' ) one = 1;
-                 full = 0;
-                 if ( a_line[i+1] == 'F' ) full = 1;
-                 if ( a_line[i+1] == 'G' ) full = 1;
-                 // FIXME: 'one' is used to merge F and G options in the future
-                 int z;
+      case 'F' : if( work_mode == WM_ARCHIVE )
+                   {
+                   s = files_list_get(FLI)->name();
+                   }
+                 else if( full )
+                   {
+                   s = files_list_get(FLI)->full_name();
+                   }
+                 else
+                   {
+                   s = files_list_get(FLI)->name();
+                   }
+                 out += shell_escape( s );
+                 break;
+
+      case 'g' : /* list selected filenames */
+      case 'G' : int z;
                  for( z = 0; z < files_list_count(); z++ )
                    {
                    TF *fi = files_list_get(z);
-                   if (  one && z != FLI ) continue; /* if one and not current -- skip */
-                   if ( !one && !fi->sel ) continue; /* if not one and not selected -- skip */
+                   if ( ! fi->sel ) continue; /* if not one and not selected -- skip */
                    if( work_mode == WM_ARCHIVE )
                      {
                      s = files_list_get(z)->name();
                      }
                    else if( full )
                      {
-                     files_list_get(z)->full_name();
+                     s = files_list_get(z)->full_name();
                      }
                    else
                      {
                      s = files_list_get(z)->name();
                      }
-                   if( !one )
-                     {
-                     // need to list files, quote them...
-                     // FIXME: this should be optional for all one and !one
-                     if ( str_find( s, "'" ) == -1 )
-                       {
-                       s = "'" + s +"' ";
-                       }
-                     else if ( str_find( s, "\"" ) == -1 )
-                       {
-                       s = "\"" + s +"\" ";
-                       }
-                     else
-                       {
-                       s = "";
-                       }
-                     }
-                   out += s;
+                   out += shell_escape( s ) + " ";
                    }
                  break;
 
@@ -116,18 +94,21 @@ while( a_line[i] )
                    s = str_file_name( s );
                  else
                    s = files_list_get(FLI)->ext();
-                 out += s;
+                 out += shell_escape( s );
                  break;
+
       case 's' : /* current file size */
                  sprintf( s, "%.0f", files_list_get(FLI)->size() );
                  out += s;
                  break;
+
       case '?' : /* prompt user for argument */
                  if (vfu_get_str( "Enter parameter:", s, HID_SHELL_PAR ))
                    out += s;
                  else
                    return 3;
                  break;
+
       case 'd' : /* prompt user for directory */
                  s = "";
                  if (vfu_get_dir_name( "Enter directory:", s, 0 ))
@@ -135,31 +116,25 @@ while( a_line[i] )
                  else
                    return 3;
                  break;
+
       case 'c' : /* current path */
                  s = work_path;
-                 #ifdef _TARGET_GO32_
-                  str_tr( s, "/", "\\" );
-                 #endif
-                 out += s;
+                 out += shell_escape( s );
                  break;
 
       case 'C' : /* startup dir */
                  s = startup_path;
-                 #ifdef _TARGET_GO32_
-                  str_tr( s, "/", "\\" );
-                 #endif
-                 out += s;
+                 out += shell_escape( s );
                  break;
 
       case 'a' : /* Archive name */
-                 out += archive_name;
+                 s = archive_name;
+                 out += shell_escape( s );
                  break;
+
       case 'A' : /* Archive path */
                  s = archive_path;
-                 #ifdef _TARGET_GO32_
-                  str_tr( s, "/", "\\" );
-                 #endif
-                 out += s;
+                 out += shell_escape( s );
                  break;
 
       case 'w' :
@@ -189,10 +164,10 @@ while( a_line[i] )
 int vfu_break_op()
 {
   if (con_kbhit())
-    if (con_getch() == 27)
+    if (con_getwch() == 27)
       {
       say2( "Press ENTER to cancel or other key to continue..." );
-      int key = con_getch();
+      int key = con_getwch();
       say2( "" );
       if ( key == 13 )
         return 1;
@@ -241,12 +216,17 @@ fsize_t vfu_update_sel_size( int one ) // used before copy/move to calc estimate
 
 /*---------------------------------------------------------------------------*/
 
-int vfu_ask( const char *prompt, const char *allowed )
+wchar_t vfu_ask( const wchar_t *prompt, const wchar_t *allowed )
 {
-  int ch = 1;
-  say1( prompt );
-  while ( strchr( allowed, ch ) == NULL ) ch = con_getch();
-  return ch;
+  wchar_t wch = 0;
+  say1( VString( prompt ) );
+  while(4)
+    {
+    wch = con_getwch();
+    if( wch == 27 ) return wch;
+    if( str_find( allowed, wch ) >= 0 ) return wch;
+    }
+  return 0;
 }
 
 /*---------------------------------------------------------------------------*/
@@ -417,7 +397,7 @@ void vfu_hist_remove( int hist_id, int index )
     }
 }
 
-int vfu_hist_menu( int x, int y, const char* title, int hist_id )
+int vfu_hist_menu( int x, int y, const wchar_t* title, int hist_id )
 {
   VString str;
 
@@ -432,7 +412,7 @@ int vfu_hist_menu( int x, int y, const char* title, int hist_id )
     str = "";
     str_add_ch( str, z < str_len( hist_menu_hotkeys ) ? hist_menu_hotkeys[z] : ' ' );
     str = str + " " + vfu_hist_get( hist_id, z );
-    mb.push( str );
+    mb.push( WString( str ) );
     }
   return vfu_menu_box( x, y, title );
 }
@@ -440,26 +420,26 @@ int vfu_hist_menu( int x, int y, const char* title, int hist_id )
 /*---------------------------------------------------------------------------*/
 
 int __vfu_get_str_hist_id; /* used to keep history id passed here... */
-void vfu_get_str_history( int key, VString &s, int &pos )
+void vfu_get_str_history( int key, WString &w, int &pos )
 {
   if ( __vfu_get_str_hist_id <= 0 ) return;
-  if ( key != KEY_NPAGE && key != KEY_PPAGE ) return;
+  if ( key != UKEY_PGUP && key != UKEY_PGDN ) return;
   con_chide();
 
-  int z = vfu_hist_menu( 5, 5, "Line History", __vfu_get_str_hist_id );
+  int z = vfu_hist_menu( 5, 5, L"Line History", __vfu_get_str_hist_id );
 
   con_cshow();
   if ( z == -1 ) return;
-  s = mb.get(z) + 2;
-  str_cut_spc( s );
-  pos = str_len( s );
+  w = WString( mb.get(z) + 2 );
+  str_cut_spc( w );
+  pos = str_len( w );
 }
 
 int vfu_get_str( const char *prompt, VString& target, int hist_id, int x, int y )
 {
   if ( x == -1 ) x = 1;
   if ( y == -1 ) y = con_max_y();
-  int len = con_max_x() - 3 - x;
+  int len = con_max_x() - x;
 
   /* FIXME: this is not correct if x and y are specified */
   if ( prompt && prompt[0] )
@@ -469,10 +449,11 @@ int vfu_get_str( const char *prompt, VString& target, int hist_id, int x, int y
   __vfu_get_str_hist_id = hist_id;
   if ( strcmp( target, "" ) == 0 && vfu_hist_get( hist_id, 0 ) )
     target = vfu_hist_get( hist_id, 0 );
-  char t[1024]; /* FIXME: can be overflowed? */
-  strcpy( t, target );
-  int r = TextInput( x, y, "", len, len, t, vfu_get_str_history );
-  target = t;
+  
+  WString www = target;
+  int r = TextInput( x, y, "", 1024, len, www, vfu_get_str_history );
+  target = www;
+
   say1( "" );
   say2( "" );
   __vfu_get_str_hist_id = 0;
@@ -483,16 +464,27 @@ int vfu_get_str( const char *prompt, VString& target, int hist_id, int x, int y
 
 /*---------------------------------------------------------------------------*/
 
-// FIXME: TODO: new function for tmp dir name etc.
 fname_t vfu_temp_filename;
 const char* vfu_temp()
 {
     strcpy( vfu_temp_filename, tmp_path + "vfu.XXXXXX" );
-    mkstemp( vfu_temp_filename );
-    unlink( vfu_temp_filename );
+    int fd = mkstemp( vfu_temp_filename );
+    if( fd >= 0 ) 
+      close( fd );
+    else
+      vfu_temp_filename[0] = 0;  
     return vfu_temp_filename;
 }
 
+fname_t vfu_temp_dirname;
+const char* vfu_temp_dir()
+{
+    strcpy( vfu_temp_dirname, tmp_path + "vfu.XXXXXX" );  
+    char* r = mkdtemp( vfu_temp_dirname );
+    if( ! r ) vfu_temp_dirname[0] = 0;
+    return vfu_temp_dirname;
+}
+
 /*---------------------------------------------------------------------------*/
 
 char* vfu_str_comma( char* target )
@@ -505,6 +497,46 @@ VString& vfu_str_comma( VString& target )
   return str_comma( target, COMMA_TYPES[opt.comma_type][0] );
 }
 
+VString vfu_str_comma( fsize_t size )
+{
+  VString str;
+  str.fi( size );
+  return str_comma( str, COMMA_TYPES[opt.comma_type][0] );
+}
+
+/*###########################################################################*/
+
+#define WCHAR_CTRL_LIST L"\u0001\u0002\u0003\u0004\u0005\u0006\u0007\u0008\u0009\u000A\u000B\u000C\u000D\u000E\u000F\u0010\u0011\u0012\u0013\u0014\u0015\u0016\u0017\u0018\u0019\u001A\u001B\u001C\u001D\u001E\u001F"
+#define WCHAR_PRNT_LIST L"\u2401\u2402\u2403\u2404\u2405\u2406\u2407\u2408\u2409\u240A\u240B\u240C\u240D\u240E\u240F\u2410\u2411\u2412\u2413\u2414\u2415\u2416\u2417\u2418\u2419\u241A\u241B\u241C\u241D\u241E\u241F"
+
+WString __vfu_translate_controls( const wchar_t *s )
+{
+  WString str = s;
+  // 00..1f --> 2400..241f, replace non-printable chars
+  str_tr( str, WCHAR_CTRL_LIST, WCHAR_PRNT_LIST );
+  return str;
+}
+
+void vfu_con_out( int x, int y, const char    *s )
+{
+  con_out( x, y, VString( __vfu_translate_controls( WString( s ) ) ) );
+}
+
+void vfu_con_out( int x, int y, const char    *s, int attr )
+{
+  con_out( x, y, VString( __vfu_translate_controls( WString( s ) ) ), attr );
+}
+
+void vfu_con_out( int x, int y, const wchar_t *s )
+{
+  con_out( x, y, VString( __vfu_translate_controls( WString( s ) ) ) );
+}
+
+void vfu_con_out( int x, int y, const wchar_t *s, int attr )
+{
+  con_out( x, y, VString( __vfu_translate_controls( WString( s ) ) ), attr );
+}
+
 /*###########################################################################*/
 
 /* eof vfuuti.cpp */
