@@ -1,10 +1,11 @@
 /****************************************************************************
- *
- * Copyright (c) 1996-2021 Vladi Belperchinov-Shabanski "Cade" 
- * http://cade.noxrun.com/  <cade@noxrun.com> <cade@bis.bg>
- *
- * SEE `README',`LICENSE' OR `COPYING' FILE FOR LICENSE AND OTHER DETAILS!
- *
+ #
+ # Copyright (c) 1996-2023 Vladi Belperchinov-Shabanski "Cade" 
+ # https://cade.noxrun.com/  <cade@noxrun.com> <cade@bis.bg>
+ # https://cade.noxrun.com/projects/vfu     https://github.com/cade-vs/vfu
+ #
+ # SEE `README',`LICENSE' OR `COPYING' FILE FOR LICENSE AND OTHER DETAILS!
+ #
  ****************************************************************************/
 
 #include "vfu.h"
@@ -24,8 +25,7 @@ int get_item_color( TF *fi )
 
   ASSERT( fi );
 
-  if (!opt.use_colors)
-    return cNORMAL; /* don't use colors -- option */
+  if ( ! opt.use_colors ) return cNORMAL; /* don't use colors */
 
   /* try to find file type color first */
 
@@ -35,11 +35,10 @@ int get_item_color( TF *fi )
   if ( str != ".." )
     {
     for ( z = cBLACK; z <= chWHITE; z++ )
-      if (str_find( ext_colors[z], str ) != -1)
+      if ( str_find( ext_colors[z], str ) != -1 )
         return z;
     }
 
-
   /* regular file, try to find extension color */
 
   str = fi->name();
@@ -54,15 +53,10 @@ int get_item_color( TF *fi )
 
   if ( opt.lower_case_ext_config ) str_low( str ); // lowercase extension
 
-  #ifdef _TARGET_GO32_
-  /* under dos/windows file names are not case sensitive */
-  str_low( str );
-  #endif
-
   if ( str != ".." )
     {
     for ( z = cBLACK; z <= chWHITE; z++ )
-      if (str_find( ext_colors[z], str ) != -1)
+      if ( str_find( ext_colors[z], str ) != -1)
         return z;
     }
 
@@ -116,7 +110,7 @@ void show_pos( int curr, int all )
 {
   char t[64];
   sprintf( t, "%5d of %5d", curr, all );
-  con_out( con_max_x() - 13, 3, t, cHEADER );
+  vfu_con_out( con_max_x() - 13, 3, t, cHEADER );
 
   if( all == 0 ) return;
 
@@ -139,7 +133,8 @@ void show_pos( int curr, int all )
 
   for( int z = 0; z < y; z++ )
     {
-    con_out( x, y1 + z, " ", ( s1 <= z and z <= s2 ) ? CONCOLOR(cBLACK,cCYAN) : cNORMAL );
+    vfu_con_out( x, y1 + z, " ", ( s1 <= z and z <= s2 ) ? CONCOLOR(cBLACK,cCYAN) : cNORMAL );
+    //vfu_con_out( x, y1 + z, ( s1 <= z and z <= s2 ) ? " " : ".", ( s1 <= z and z <= s2 ) ? CONCOLOR(cBLACK,cCYAN) : cCYAN );
     }
 }
 
@@ -147,16 +142,15 @@ void show_pos( int curr, int all )
 
 void vfu_drop_all_views()
 {
-  int z = 0;
-  for( z = 0; z < files_list_count(); z++ ) files_list_get(z)->drop_view();
-  do_draw += 1;
+  for( int z = 0; z < files_list_count(); z++ ) files_list_get( z )->drop_view();
+  do_draw++;
 }
 
 /*#######################################################################*/
 
 void vfu_draw( int n )
 {
-  if ( n < FLP || n > FLP + FPS )
+  if ( n < FLP || n > FLP + FLPS )
     return; /* we are out of screen -- don't draw */
 
   TF* fi = files_list_get(n);
@@ -166,7 +160,7 @@ void vfu_draw( int n )
   if ( fi->sel )
     {
     str_set_ch( view, sel_mark_pos, '#' );
-    c = CONCOLOR(cBLACK,cWHITE);
+    c = CONCOLOR( cBLACK, cWHITE );
     }
   if ( n == FLI )
     {
@@ -181,13 +175,13 @@ void vfu_draw( int n )
     //str_replace( view, " ", "-" );
     //c = CONCOLOR(cWHITE,cBLUE);
     }
-  con_out( 1, n - FLP + 4, view, c );
+  vfu_con_out( 1, n - FLP + 4, view, c );
   //  con_ce( c );
 }
 
 /*#######################################################################*/
 
-extern const char *FTIMETYPE[]; /* in vfuopt.cpp */
+extern const wchar_t *FTIMETYPE[]; /* in vfuopt.cpp */
 void vfu_redraw() /* redraw file list and header */
 {
   fname_t t;
@@ -195,19 +189,19 @@ void vfu_redraw() /* redraw file list and header */
 
   str  = "Mask: ";
   str += files_mask;
-  con_out(1,1,str,cINFO);
-  con_ce(cINFO);
+  vfu_con_out( 1, 1, VString(str), cINFO );
+  con_ce( cINFO );
   if ( work_mode == WM_ARCHIVE )
-    con_out( con_max_x()-34, 1, " [-ARCHIVE-] ", cWARNING );
-  con_out(con_max_x()-17,1,"Press H for help",cINFO);
-  con_out(con_max_x()-20,1,"VFU " VFU_VERSION " <H> for help",cINFO);
+    vfu_con_out( con_max_x()-34, 1, " [-ARCHIVE-] ", cWARNING );
+  vfu_con_out( con_max_x()-17, 1, "Press H for help", cINFO);
+  vfu_con_out( con_max_x()-20, 1, "VFU " VFU_VERSION " <H> for help",cINFO);
 
   str = "Path: ";
   str += work_path;
   if ( work_mode == WM_ARCHIVE )
     str += "[" + archive_name + "]/" + archive_path; /* NOTE: to simulate root dir visually */
   str = str_dot_reduce( str, con_max_x()-1 );
-  con_out( 1, 2, str, cINFO );
+  vfu_con_out( 1, 2, str, cINFO );
   con_ce( cINFO );
 
   str = "";
@@ -218,14 +212,14 @@ void vfu_redraw() /* redraw file list and header */
   if ( opt.sort_order == 'A' ) str = "MODE";
   if ( opt.sort_order == 'O' ) str = "OWNER";
   if ( opt.sort_order == 'G' ) str = "GROUP";
-  if ( opt.sort_order == 'T' ) str = "MTIME";
-  if ( opt.sort_order == 'H' ) str = "CTIME";
-  if ( opt.sort_order == 'C' ) str = "ATIME";
-  if ( opt.sort_order == 'S' ) str = "SIZE";
+  if ( opt.sort_order == 'T' ) str = "MTiME";
+  if ( opt.sort_order == 'H' ) str = "CTiME";
+  if ( opt.sort_order == 'C' ) str = "ATiME";
+  if ( opt.sort_order == 'S' ) str = "SiZE";
   if ( opt.sort_order == 'Y' ) str = "TYPE";
   str += opt.sort_direction == 'A' ? "+" : "-";
   str = "(SORT:" + str + ")";
-  con_out( con_max_x() - str_len( str ) + 1, 2, str, cHEADER );
+  vfu_con_out( con_max_x() - str_len( str ) + 1, 2, str, cHEADER );
 
   str = "";
 
@@ -234,43 +228,44 @@ void vfu_redraw() /* redraw file list and header */
   if (opt.sort_order == 'D') opt.sort_order = 'T'; /* hack anyway */
   if (!opt.long_name_view)
     {
-    if ( opt.f_mode  ) spos += sprintf( spos, "%10s ", MODE_STRING );
+    //if ( opt.f_mode  ) spos += sprintf( spos, "%10s ", MODE_STRING );
+    if ( opt.f_mode  ) spos += sprintf( spos, "MODE       " );
     if ( opt.f_owner ) spos += sprintf( spos, "   OWNER " );
     if ( opt.f_group ) spos += sprintf( spos, "   GROUP " );
-    if ( opt.f_time  ) spos += sprintf( spos, "%s  TiME ", FTIMETYPE[opt.f_time_type] );
+    if ( opt.f_time  ) spos += sprintf( spos, "  %s TiME ", VString( FTIMETYPE[opt.f_time_type] ).data() );
     if ( opt.f_size  ) spos += sprintf( spos, "          SiZE " );
     };
   if ( opt.f_mode + opt.f_owner + opt.f_group + opt.f_time + opt.f_size + opt.f_type == 0 )
     opt.f_type = 1; /* a hack really :) if all fields are off -- turn on type one */
   if ( opt.f_type || opt.long_name_view ) 
-    spos += sprintf( spos, "TP" );
+    spos += sprintf( spos, "TY" );
   tag_mark_pos = strlen( t );
   sel_mark_pos = tag_mark_pos + 2;
 
-  /* spos += */ sprintf( spos, "  #NAME    %s",
-                         opt.long_name_view ? "( long name view )" : "" );
+  /* spos += */ sprintf( spos, "  #NAME  %s",
+                         opt.long_name_view ? "( long name view, press 0 to disable )" : "" );
 
   str_pad( t, - con_max_x() );
   str_sleft( t, con_max_x() );
 
-  con_out(1,3, t, cHEADER );
+  vfu_con_out( 1, 3, t, cHEADER );
   show_pos( FLI+1, files_list_count() );
 
   int z;
 
-  for ( z = 0; z < FPS; z++ )
+  for ( z = 0; z < FLPS; z++ )
     {
     ASSERT( FLP + z >= 0 );
     if ( FLP + z >= files_list_count() || files_list_is_empty( FLP + z ) )
       {
-      con_out( 1, z + 4, "~", cPLAIN );
+      vfu_con_out( 1, z + 4, "~", cPLAIN );
       con_ce( cPLAIN );
       }
     else
       vfu_draw( FLP + z );
     }
   if ( files_list_count() <= 0 )
-    con_out( ( con_max_x() - 20 ) / 2, 10, " *** No files found *** ", cHEADER);
+    vfu_con_out( ( con_max_x() - 20 ) / 2, 10, " *** No files found *** ", cHEADER);
 
 }
 
@@ -323,18 +318,22 @@ void vfu_redraw_status() /* redraw bottom status, total,free,selected... */
   str_pad( tmp, 14 );
   s2 += tmp;
 
-  tmp = fs_block_size; str_pad( tmp,5 ); s2 += " [" + tmp + "]";
-  sprintf( tmp,  " %s.%s@%s ", 
-                 user_id_str.data(), 
-                 group_id_str.data(),
-                 host_name_str.data() );
+  tmp = fs_block_size; 
+  str_pad( tmp, 5 ); 
+  s2 += " [" + tmp + "]";
+  
+  
+  tmp = " ";
+  tmp = tmp + user_id_str + "." + group_id_str + "@" + host_name_str;
+  str_padw( tmp, con_max_x() - str_len( s2 ) - 1 );
+  
   s2 += tmp;
 
   str_pad( s1, - con_max_x() );
   str_pad( s2, - con_max_x() );
 
-  con_out( 1, con_max_y()-3, s1, cINFO2 );
-  con_out( 1, con_max_y()-2, s2, cINFO2 );
+  vfu_con_out( 1, con_max_y()-3, s1, cINFO2 );
+  vfu_con_out( 1, con_max_y()-2, s2, cINFO2 );
 }
 
 /*#######################################################################*/
@@ -377,8 +376,8 @@ void vfu_nav_down()
 
 void vfu_nav_ppage()
 {
-if ( files_list_count() == 0 ) return;
-if ( FLP == 0 && FLI == 0 ) return;
+  if ( files_list_count() == 0 ) return;
+  if ( FLP == 0 && FLI == 0 ) return;
 
   int old_fli = FLI;
   int old_flp = FLP;
@@ -397,7 +396,7 @@ if ( FLP == 0 && FLI == 0 ) return;
 void vfu_nav_npage()
 {
   if ( files_list_count() == 0 ) return;
-  if ( FLP >= files_list_count() - FPS && FLI == files_list_count() - 1 ) return;
+  if ( FLP >= files_list_count() - FLPS && FLI == files_list_count() - 1 ) return;
 
   int old_fli = FLI;
   int old_flp = FLP;
@@ -427,16 +426,16 @@ void vfu_nav_home()
       z++;
       }
     if( z < files_list_count() )
-      FGO( z );
+      FLGO( z );
     }
   else
-    FGO( 0 );
+    FLGO( 0 );
   vfu_nav_update_pos();
   do_draw = 1;
 /*
   if ( files_list_count() == 0 ) return;
   if ( FLI == 0 ) return;
-  FGO( 0 );
+  FLGO( 0 );
   vfu_nav_update_pos();
   do_draw = 1;
 */  
@@ -458,16 +457,16 @@ void vfu_nav_end()
       z--;
       }
     if( z >= 0 )
-      FGO( z );
+      FLGO( z );
     }
   else
-    FGO( files_list_count() - 1 );
+    FLGO( files_list_count() - 1 );
   vfu_nav_update_pos();
   do_draw = 1;
 /*
   if ( files_list_count() == 0 ) return;
   if ( FLI >= files_list_count() - 1 ) return;
-  FGO( files_list_count() - 1 );
+  FLGO( files_list_count() - 1 );
   vfu_nav_update_pos();
   do_draw = 1;
 */
@@ -507,9 +506,9 @@ void vfu_nav_select()
 void vfu_nav_update_pos()
 {
  ASSERT( files_list_count() >= 0 );
- if ( FLI < 0 ) FGO( 0 );
- if ( files_list_count() == 0 ) FGO( 0 );
- if ( files_list_count() > 0 && FLI > files_list_count() - 1 ) FGO( files_list_count() - 1 );
+ if ( FLI < 0 ) FLGO( 0 );
+ if ( files_list_count() == 0 ) FLGO( 0 );
+ if ( files_list_count() > 0 && FLI > files_list_count() - 1 ) FLGO( files_list_count() - 1 );
 }
 
 /* eof vfuview.cpp */
