@@ -1,7 +1,7 @@
 /****************************************************************************
  *
- * Copyright (c) 1996-2020 Vladi Belperchinov-Shabanski "Cade" 
- * http://cade.datamax.bg/  <cade@biscom.net> <cade@bis.bg> <cade@datamax.bg>
+ *  Copyright (c) 1996-2022 Vladi Belperchinov-Shabanski "Cade" 
+ *  http://cade.noxrun.com/  <cade@noxrun.com> <cade@bis.bg> <cade@cpan.org>
  *
  * SEE `README',`LICENSE' OR `COPYING' FILE FOR LICENSE AND OTHER DETAILS!
  *
@@ -322,14 +322,8 @@ int make_path( const char *s, long mode )
 
 char* expand_path( const char *src, char *dest )
 {
-  #ifdef _TARGET_UNIX_
-  if (!realpath( src, dest))
+  if( realpath( src, dest ) == 0 )
     strcpy( dest, src );
-  #endif
-
-  #ifdef _TARGET_GO32_
-  _fixpath( src, dest );
-  #endif
 
   return dest;
 }
@@ -341,6 +335,50 @@ VString expand_path( const char* src )
   return dest;
 }
 
+/*****************************************************************************
+**
+** shell_escape() escapes shell special characters    '\"`&;*()[]{}!^:
+**
+** this seems to work fine with bash, zsh, csh, tcsh, fish, ksh, dash
+*****************************************************************************/
+
+const char* shell_special_chars = "'\"\\`;&*()[]{}!^:?$<> ";
+
+char* shell_escape( const char *src, char *dest )
+{
+  int sl = strlen( src );
+  int z = 0;
+  for( int i = 0; i < sl; i++ )
+    {      
+    if( strchr( shell_special_chars, src[i] ) )
+      dest[z++] = '\\';
+    dest[z++] = src[i];
+    }
+  dest[z] = 0;  
+  return dest;  
+}
+
+VString& shell_escape( VString &dest )
+{
+  int sl = strlen( dest );
+  for( int i = 0; i < sl; i++ )
+    {      
+    if( ! strchr( shell_special_chars, dest[i] ) ) continue;
+    str_ins_ch( dest, i, '\\' );
+    sl++;
+    i++;
+    }
+  return dest;  
+}
+
+VString shell_escape( const char* src )
+{
+  VString dest;
+  dest = src;
+  shell_escape( dest );
+  return dest;
+}
+
 /*****************************************************************************
 **
 ** dosstat() is fast stat() designed for DOS FAT filesystems under DJGPP.
