@@ -1,9 +1,9 @@
 /****************************************************************************
  *
- * Copyright (c) 1996-2020 Vladi Belperchinov-Shabanski "Cade" 
- * http://cade.datamax.bg/  <cade@biscom.net> <cade@bis.bg> <cade@datamax.bg>
+ *  Copyright (c) 1996-2022 Vladi Belperchinov-Shabanski "Cade" 
+ *  http://cade.noxrun.com/  <cade@noxrun.com> <cade@bis.bg> <cade@cpan.org>
  *
- * SEE `README',`LICENSE' OR `COPYING' FILE FOR LICENSE AND OTHER DETAILS!
+ *  SEE `README',`LICENSE' OR `COPYING' FILE FOR LICENSE AND OTHER DETAILS!
  *
  ****************************************************************************/
 
@@ -58,7 +58,7 @@ typedef unsigned long int crc32_t;
 #define CRC32NULL (0xffffffff)
 
 /* should start with `0xffffffff' for `crc' and result is crc = ~crc; */
-crc32_t update_crc32( const unsigned char octet, const crc32_t crc );
+inline crc32_t update_crc32( const unsigned char octet, const crc32_t crc );
 
 crc32_t mem_crc32( const void* buff, int size );
 crc32_t str_crc32( const char *s );
@@ -124,6 +124,16 @@ int make_path( const char *s,
 char* expand_path( const char *src, char *dest );
 VString expand_path( const char* src );
 
+/*****************************************************************************
+**
+** shell_escape() escapes shell special characters
+**
+*****************************************************************************/
+
+char*    shell_escape( const char *src, char *dest );
+VString& shell_escape( VString &dest );
+VString  shell_escape( const char* src );
+
 /*****************************************************************************
 **
 ** dosstat() is fast stat() designed for DOS FAT filesystems under DJGPP.
